from unittest import TestCase

from musicscore.musicxml.types.complextypes.attributes import Clef
from musicscore.musicxml.types.complextypes.clef import Sign


class Test(TestCase):
    def setUp(self) -> None:
        self.clef = Clef()
        self.clef.add_child(Sign('F'))

    def test_1(self):
        clef = self.clef

        expected = '''<clef>
  <sign>F</sign>
</clef>
'''
        actual = clef.to_string()
        self.assertEqual(expected, actual)

    def test_2(self):
        clef = self.clef
        clef.number = 2
        expected = '''<clef number="2">
  <sign>F</sign>
</clef>
'''
        actual = clef.to_string()
        self.assertEqual(expected, actual)

    def test_3(self):
        clef = self.clef
        clef.after_barline = 'yes'
        expected = '''<clef after-barline="yes">
  <sign>F</sign>
</clef>
'''
        actual = clef.to_string()
        self.assertEqual(expected, actual)

    def test_4(self):
        clef = self.clef
        clef.size = 'large'
        expected = '''<clef size="large">
  <sign>F</sign>
</clef>
'''
        actual = clef.to_string()
        self.assertEqual(expected, actual)

    def test_5(self):
        clef = self.clef
        clef.additional = 'no'
        expected = '''<clef additional="no">
  <sign>F</sign>
</clef>
'''
        actual = clef.to_string()
        self.assertEqual(expected, actual)

    def test_6(self):
        clef = self.clef
        clef.number = 2
        clef.additional = 'no'
        clef.size = 'large'
        clef.after_barline = 'yes'
        expected = '''<clef number="2" additional="no" size="large" after-barline="yes">
  <sign>F</sign>
</clef>
'''
        actual = clef.to_string()
        self.assertEqual(expected, actual)

    # def test_7(self):
    #     clef = Clef(number=2)



import os
from unittest import TestCase

from musicscore.musictree.treemeasure import TreePart
from musicscore.musicxml.groups.musicdata import Attributes

path = str(os.path.abspath(__file__).split('.')[0])


class Test(TestCase):

    def setUp(self) -> None:
        self.part = TreePart(id='test_part')

    def test_1(self):
        self.assertIsNone(self.part.staves)

    def test_2(self):
        p = self.part
        p.staves = 1
        expected = 1
        actual = p.staves
        self.assertEqual(expected, actual)

    def test_3(self):
        p = self.part
        p.staves = 1
        p.staves = 2
        expected = 2
        actual = p.staves
        self.assertEqual(expected, actual)



from musicscore.musictree.treechord import TreeChord
from musicxmlunittest import XMLTestCase


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.chord = TreeChord()

    def test_1(self):
        self.chord.add_words('1')
        copied = self.chord.__deepcopy__()
        print(copied._current_children[0]._current_children[0].__dict__)


import os
from unittest import TestCase

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treeclef import BASS_CLEF
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase
from tests.score_templates.xml_test_score import TestScore

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):

    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        xml_path = path + '_test_1.xml'
        sf = SimpleFormat(quarter_durations=4, midis=0)
        sf.to_stream_voice().add_to_score(self.score)
        self.score.get_measure(1).get_part(1).add_clef(BASS_CLEF)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treeclef import TreeClef, BASS_CLEF
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase
from tests.score_templates.xml_test_score import TestScore

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):

    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        clef = TreeClef(number=2)
        sf = SimpleFormat(quarter_durations=[1, 1])
        sf.chords[0].add_clef(clef)
        sf.to_stream_voice().add_to_score(self.score)

        xml_path = path + '_test_1.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)

    def test_2(self):
        sf = SimpleFormat(quarter_durations=[1, 1])
        sf.chords[0].add_clef(BASS_CLEF)
        sf.to_stream_voice().add_to_score(self.score)

        xml_path = path + '_test_2.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



"""
<xs:complexType name="pedal">
        <xs:attribute name="type" type="pedal-type" use="required"/>
        <xs:attribute name="number" type="number-level"/>
        <xs:attribute name="line" type="yes-no"/>
        <xs:attribute name="sign" type="yes-no"/>
        <xs:attribute name="abbreviated" type="yes-no"/>
        <xs:attributeGroup ref="print-style-align"/>
        <xs:attributeGroup ref="optional-unique-id"/>
    </xs:complexType>
"""
from musicscore.musicxml.attributes.optional_unique_id import OptionalUniqueId
from musicscore.musicxml.attributes.printstyle import PrintStyleAlign
from musicscore.musicxml.types.complextypes.complextype import ComplexType
from musicscore.musicxml.types.simple_type import TypeNumberLevel, TypeYesNo, TypePedalType


class ComplexTypePedal(ComplexType, PrintStyleAlign, OptionalUniqueId):
    """The pedal type represents piano pedal marks. In MusicXML 3.1 this includes sostenuto as well as damper pedal
    marks. The line attribute is yes if pedal lines are used. The sign attribute is yes if Ped, Sost, and * signs are
    used. For MusicXML 2.0 compatibility, the sign attribute is yes by default if the line attribute is no, and is no
    by default if the line attribute is yes. If the sign attribute is set to yes and the type is start or sostenuto,
    the abbreviated attribute is yes if the short P and S signs are used, and no if the full Ped and Sost signs are
    used. It is no by default. Otherwise the abbreviated attribute is ignored.

    The change and continue types are used when the line attribute is yes. The change type indicates a pedal lift and
    retake indicated with an inverted V marking. The continue type allows more precise formatting across system breaks
    and for more complex pedaling lines. The alignment attributes are ignored if the line attribute is yes."""

    def __init__(self, tag, type, line='no', *args, **kwargs):
        super().__init__(tag=tag, *args, **kwargs)
        self.line = line
        self.type = type

    @property
    def type(self):
        return self.get_attribute('type')

    @type.setter
    def type(self, value):
        if value is None:
            self.remove_attribute('type')
        else:
            TypePedalType(value)
            self._ATTRIBUTES.insert(0, 'type')
            self.set_attribute('type', value)

    @property
    def number(self):
        return self.get_attribute('number')

    @number.setter
    def number(self, value):
        if value is None:
            self.remove_attribute('number')
        else:
            TypeNumberLevel(value)
            self._ATTRIBUTES.insert(0, 'number')
            self.set_attribute('number', value)

    @property
    def line(self):
        return self.get_attribute('line')

    @line.setter
    def line(self, value):
        if value is None:
            self.remove_attribute('line')
        else:
            TypeYesNo(value)
            self._ATTRIBUTES.insert(0, 'line')
            self.set_attribute('line', value)

    @property
    def sign(self):
        return self.get_attribute('sign')

    @sign.setter
    def sign(self, value):
        if value is None:
            self.remove_attribute('sign')
        else:
            TypeYesNo(value)
            self._ATTRIBUTES.insert(0, 'sign')
            self.set_attribute('sign', value)

    @property
    def abbreviated(self):
        return self.get_attribute('abbreviated')

    @abbreviated.setter
    def abbreviated(self, value):
        if value is None:
            self.remove_attribute('abbreviated')
        else:
            TypeYesNo(value)
            self._ATTRIBUTES.insert(0, 'abbreviated')
            self.set_attribute('abbreviated', value)



import os

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_1(self):
        simple_format = SimpleFormat(quarter_durations=4)
        simple_format.to_stream_voice().add_to_score(self.score)
        part = self.score.get_measure(1).get_part(1)
        expected = part.id
        actual = part.parent_score_part.id
        self.assertEqual(expected, actual)



import os
from unittest import TestCase

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treeinstruments import Violin
from musicscore.musictree.treepart import TreePart, TreePartStaff, TreePartVoice
from musicscore.musictree.treescoretimewise import TreeScoreTimewise

path = str(os.path.abspath(__file__).split('.')[0])


class Test(TestCase):
    def setUp(self) -> None:
        self.part = TreePart(id='test_part')

    def test_1(self):
        self.assertIsNone(self.part.staves)

    def test_2(self):
        self.part.add_tree_part_staff(TreePartStaff(2))
        expected = 2
        actual = list(self.part.tree_part_staves)[0]
        self.assertEqual(expected, actual)

    def test_3(self):
        self.part.set_tree_part_staff(staff_number=3)
        expected = 3
        actual = list(self.part.tree_part_staves)[0]
        self.assertEqual(expected, actual)

    def test_4(self):
        tps = self.part.set_tree_part_staff(2)
        tps.add_tree_part_voice(TreePartVoice(2))
        expected = 2
        actual = list(self.part.tree_part_staves[2].tree_part_voices)[0]
        self.assertEqual(expected, actual)

    def test_5(self):
        tree_part_staff = self.part.set_tree_part_staff(1)
        tree_part_staff.add_tree_part_voice(TreePartVoice(2))
        with self.assertRaises(AttributeError):
            tree_part_staff.add_tree_part_voice(TreePartVoice(2))

    def test_6(self):
        tps = self.part.set_tree_part_staff(1)
        tpv_1 = tps.set_tree_part_voice(1)
        tpv_2 = tps.set_tree_part_voice(2)
        expected = [tpv_1, tpv_2]
        actual = self.part.tree_part_voices
        self.assertEqual(expected, actual)




from pathlib import Path

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise
from musicxmlunittest import XMLTestCase

path = Path(__file__)


class TestAddPedal(XMLTestCase):
    def setUp(self) -> None:
        self.score = TreeScoreTimewise()

    def test_imply(self):
        xml_path = path.parent.joinpath(path.stem + '_imply.xml')
        sf = SimpleFormat(quarter_durations=[1, 1, 1, 1])
        sf.chords[0].add_pedal('start')
        sf.chords[2].add_pedal('stop')
        sf.to_stream_voice().add_to_score(self.score)
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



from musicscore.musicxml.attributes.color import Color
from musicscore.musicxml.attributes.placement import Placement
from musicscore.musicxml.attributes.position import Position
from musicscore.musicxml.attributes.trillsound import TrillSound
from musicscore.musicxml.types.complextypes.complextype import ComplexType
from musicscore.musicxml.types.simple_type import TypeStartStopContinue, TypeNumberLevel


class ComplexTypeWavyLine(ComplexType, Position, Placement, Color, TrillSound):
    """
    Wavy lines are one way to indicate trills. When used with a barline element, they should always have
    type="continue" set.
    """

    def __init__(self, tag, type, number=1, *args, **kwargs):
        super().__init__(tag=tag, *args, **kwargs)
        self.type = type
        self.number = number

    @property
    def number(self):
        return self.get_attribute('number')

    @number.setter
    def number(self, value):
        if value is None:
            self.remove_attribute('number')
        else:
            TypeNumberLevel(value)
            self._ATTRIBUTES.insert(0, 'number')
            self.set_attribute('number', value)

    @property
    def type(self):
        return self.get_attribute('type')

    @type.setter
    def type(self, value):
        if value is None:
            self.remove_attribute('type')
        else:
            TypeStartStopContinue(value)
            self._ATTRIBUTES.insert(0, 'type')
            self.set_attribute('type', value)
